@@ -559,28 +559,6 @@ ktest_make_sample_ad_kdcissued(krb5_ad_kdcissued *p)
     ktest_make_sample_authorization_data(&p->elements);
 }
 
-void
-ktest_make_sample_ad_signedpath_data(krb5_ad_signedpath_data *p)
-{
-    ktest_make_sample_principal(&p->client);
-    p->authtime = SAMPLE_TIME;
-    p->delegated = ealloc(2 * sizeof(krb5_principal));
-    ktest_make_sample_principal(&p->delegated[0]);
-    p->delegated[1] = NULL;
-    ktest_make_sample_authorization_data(&p->authorization_data);
-    ktest_make_sample_pa_data_array(&p->method_data);
-}
-
-void
-ktest_make_sample_ad_signedpath(krb5_ad_signedpath *p)
-{
-    p->enctype = 1;
-    ktest_make_sample_checksum(&p->checksum);
-    p->delegated = ealloc(2 * sizeof(krb5_principal));
-    p->delegated[1] = NULL;
-    ktest_make_sample_pa_data_array(&p->method_data);
-}
-
 void
 ktest_make_sample_iakerb_header(krb5_iakerb_header *ih)
 {
@@ -722,9 +700,7 @@ ktest_make_sample_pk_authenticator(krb5_pk_authenticator *p)
     p->cusec = SAMPLE_USEC;
     p->ctime = SAMPLE_TIME;
     p->nonce = SAMPLE_NONCE;
-    ktest_make_sample_checksum(&p->paChecksum);
-    /* We don't encode the checksum type, only the contents. */
-    p->paChecksum.checksum_type = 0;
+    ktest_make_sample_data(&p->paChecksum);
     p->freshnessToken = ealloc(sizeof(krb5_data));
     ktest_make_sample_data(p->freshnessToken);
 }
@@ -750,13 +726,6 @@ ktest_make_sample_algorithm_identifier_no_params(krb5_algorithm_identifier *p)
     p->parameters = empty_data();
 }
 
-static void
-ktest_make_sample_subject_pk_info(krb5_subject_pk_info *p)
-{
-    ktest_make_sample_algorithm_identifier(&p->algorithm);
-    ktest_make_sample_data(&p->subjectPublicKey);
-}
-
 static void
 ktest_make_sample_external_principal_identifier(
     krb5_external_principal_identifier *p)
@@ -806,8 +775,8 @@ void
 ktest_make_sample_auth_pack(krb5_auth_pack *p)
 {
     ktest_make_sample_pk_authenticator(&p->pkAuthenticator);
-    p->clientPublicValue = ealloc(sizeof(krb5_subject_pk_info));
-    ktest_make_sample_subject_pk_info(p->clientPublicValue);
+    /* Need a valid DER encoding here; this is the OCTET STRING "pvalue". */
+    krb5_data_parse(&p->clientPublicValue, "\x04\x06" "pvalue");
     p->supportedCMSTypes = ealloc(3 * sizeof(krb5_algorithm_identifier *));
     p->supportedCMSTypes[0] = ealloc(sizeof(krb5_algorithm_identifier));
     ktest_make_sample_algorithm_identifier(p->supportedCMSTypes[0]);
@@ -1097,7 +1066,7 @@ ktest_destroy_keyblock(krb5_keyblock **kb)
 void
 ktest_empty_authorization_data(krb5_authdata **ad)
 {
-    int i;
+    size_t i;
 
     if (*ad != NULL) {
         for (i=0; ad[i] != NULL; i++)
@@ -1126,7 +1095,7 @@ ktest_destroy_authdata(krb5_authdata **ad)
 void
 ktest_empty_pa_data_array(krb5_pa_data **pad)
 {
-    int i;
+    size_t i;
 
     for (i=0; pad[i] != NULL; i++)
         ktest_destroy_pa_data(&pad[i]);
@@ -1163,7 +1132,7 @@ ktest_destroy_address(krb5_address **a)
 void
 ktest_empty_addresses(krb5_address **a)
 {
-    int i;
+    size_t i;
 
     for (i=0; a[i] != NULL; i++)
         ktest_destroy_address(&a[i]);
@@ -1202,7 +1171,7 @@ ktest_destroy_sequence_of_integer(long **soi)
 void
 ktest_destroy_sequence_of_ticket(krb5_ticket ***sot)
 {
-    int i;
+    size_t i;
 
     for (i=0; (*sot)[i] != NULL; i++)
         ktest_destroy_ticket(&(*sot)[i]);
@@ -1249,7 +1218,7 @@ ktest_destroy_etype_info_entry(krb5_etype_info_entry *i)
 void
 ktest_destroy_etype_info(krb5_etype_info_entry **info)
 {
-    int i;
+    size_t i;
 
     for (i = 0; info[i] != NULL; i++)
         ktest_destroy_etype_info_entry(info[i]);
@@ -1400,7 +1369,7 @@ ktest_destroy_cred_info(krb5_cred_info **ci)
 void
 ktest_destroy_sequence_of_cred_info(krb5_cred_info ***soci)
 {
-    int i;
+    size_t i;
 
     for (i = 0; (*soci)[i] != NULL; i++)
         ktest_destroy_cred_info(&(*soci)[i]);
@@ -1442,7 +1411,7 @@ ktest_empty_cred(krb5_cred *c)
 void
 ktest_destroy_last_req(krb5_last_req_entry ***lr)
 {
-    int i;
+    size_t i;
 
     if (*lr) {
         for (i=0; (*lr)[i] != NULL; i++)
@@ -1531,39 +1500,6 @@ ktest_empty_ad_kdcissued(krb5_ad_kdcissued *p)
     ktest_destroy_authorization_data(&p->elements);
 }
 
-void
-ktest_empty_ad_signedpath_data(krb5_ad_signedpath_data *p)
-{
-    int i;
-
-    ktest_destroy_principal(&p->client);
-    if (p->delegated != NULL) {
-        for (i = 0; p->delegated[i] != NULL; i++) {
-            krb5_principal princ = p->delegated[i];
-            ktest_destroy_principal(&princ);
-        }
-        free(p->delegated);
-    }
-    ktest_destroy_pa_data_array(&p->method_data);
-    ktest_destroy_authorization_data(&p->authorization_data);
-}
-
-void
-ktest_empty_ad_signedpath(krb5_ad_signedpath *p)
-{
-    int i;
-
-    free(p->checksum.contents);
-    if (p->delegated != NULL) {
-        for (i = 0; p->delegated[i] != NULL; i++) {
-            krb5_principal princ = p->delegated[i];
-            ktest_destroy_principal(&princ);
-        }
-        free(p->delegated);
-    }
-    ktest_destroy_pa_data_array(&p->method_data);
-}
-
 void
 ktest_empty_iakerb_header(krb5_iakerb_header *p)
 {
@@ -1666,19 +1602,11 @@ ktest_empty_pa_otp_req(krb5_pa_otp_req *p)
 static void
 ktest_empty_pk_authenticator(krb5_pk_authenticator *p)
 {
-    ktest_empty_checksum(&p->paChecksum);
-    p->paChecksum.contents = NULL;
+    ktest_empty_data(&p->paChecksum);
     krb5_free_data(NULL, p->freshnessToken);
     p->freshnessToken = NULL;
 }
 
-static void
-ktest_empty_subject_pk_info(krb5_subject_pk_info *p)
-{
-    ktest_empty_algorithm_identifier(&p->algorithm);
-    ktest_empty_data(&p->subjectPublicKey);
-}
-
 static void
 ktest_empty_external_principal_identifier(
     krb5_external_principal_identifier *p)
@@ -1728,11 +1656,7 @@ ktest_empty_auth_pack(krb5_auth_pack *p)
     krb5_data **d;
 
     ktest_empty_pk_authenticator(&p->pkAuthenticator);
-    if (p->clientPublicValue != NULL) {
-        ktest_empty_subject_pk_info(p->clientPublicValue);
-        free(p->clientPublicValue);
-        p->clientPublicValue = NULL;
-    }
+    ktest_empty_data(&p->clientPublicValue);
     if (p->supportedCMSTypes != NULL) {
         for (ai = p->supportedCMSTypes; *ai != NULL; ai++) {
             ktest_empty_algorithm_identifier(*ai);
@@ -1783,7 +1707,7 @@ void ktest_empty_pkinit_supp_pub_info(krb5_pkinit_supp_pub_info *p)
 
 #ifdef ENABLE_LDAP
 void
-ktest_empty_ldap_seqof_key_data(krb5_context ctx, ldap_seqof_key_data *p)
+ktest_empty_ldap_seqof_key_data(ldap_seqof_key_data *p)
 {
     int i;
 
@@ -1831,7 +1755,7 @@ ktest_empty_cammac(krb5_cammac *p)
 void
 ktest_empty_secure_cookie(krb5_secure_cookie *p)
 {
-    ktest_empty_pa_data_array(p->data);
+    ktest_destroy_pa_data_array(&p->data);
 }
 
 void
